package nl.iobyte.themeparkconnector.api.client;

import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.message.MessageKey;
import nl.iobyte.themeparkconnector.api.network.packet.KickClientPacket;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class ClientConnection {

    //General
    private final UUID uuid;
    private final String name;

    //Connect
    private boolean waitingForToken = false;
    private boolean isConnected = false;
    private String usedToken = null;
    private String attraction;
    private final List<Runnable> onConnect = new ArrayList<>();
    private final List<Runnable> onDisconnect = new ArrayList<>();
    private final Publisher publisher;

    public ClientConnection(Player player) {
        this.uuid = player.getUniqueId();
        this.name = player.getName();
        this.publisher = new Publisher(this);
    }

    /**
     * Get Name of Client
     * @return Client Name
     */
    public String getOwnerName() {
        return name;
    }

    /**
     * Get UUID of Client
     * @return Client UUID
     */
    public UUID getOwnerUUID() {
        return uuid;
    }

    /**
     * Get Player
     * @return Player instance
     */
    public Player getPlayer() {
        return Bukkit.getPlayer(getOwnerUUID());
    }

    /**
     * Check if client is connected
     * @return connection boolean
     */
    public boolean isConnected() {
        return isConnected;
    }

    /**
     * When client connection is opened
     */
    public void onConnect() {
        if(isConnected)
            return;

        isConnected = true;
        MessageKey.CLIENT_CONNECTION_OPEN.send(getPlayer());
        for(Runnable runnable : onConnect)
            runnable.run();
    }

    /**
     * When client connection is closed
     */
    public void onDisconnect() {
        if(!isConnected)
            return;

        isConnected = false;
        usedToken = null;
        ThemeParkConnector.getInstance().getAPI().getOperatorService().stopOperating(uuid);
        MessageKey.CLIENT_CONNECTION_CLOSED.send(getPlayer());
        for(Runnable runnable : onDisconnect)
            runnable.run();
    }

    /**
     * Get token used to connect
     * @return String
     */
    public String getUsedToken() {
        return usedToken;
    }

    /**
     * Set token used to connect
     * @param token String
     */
    public void setUsedToken(String token) {
        usedToken = token;
    }

    /**
     * Get if client is waiting for token
     * @return boolean
     */
    public boolean isWaitingForToken() {
        return waitingForToken;
    }

    /**
     * Set if client is waiting for token
     * @param waitingForToken boolean
     */
    public void setWaitingForToken(boolean waitingForToken) {
        this.waitingForToken = waitingForToken;
    }

    /**
     * Set if client is waiting for token
     * @param waitingForToken boolean
     */
    public void setWaitingForToken(boolean waitingForToken, String attraction) {
        setWaitingForToken(waitingForToken);
        this.attraction = attraction;
    }

    public void publishURL() {
        publisher.startClientSession(attraction);
    }

    /**
     * Try generating url for client
     */
    public void publishURL(String id) {
        publisher.startClientSession(id);
    }

    /**
     * Close the clients web client
     */
    public void kick() {
        ThemeParkConnector.getInstance().getAPI().getNetworkingService().send(this, new KickClientPacket());
        ThemeParkConnector.getInstance().getAPI().getOperatorService().stopOperating(uuid);
    }

    /**
     * When ClientConnection gets removed
     */
    public void remove() {
        ThemeParkConnector.getInstance().getAPI().getNetworkingService().send(this, new KickClientPacket());
        ThemeParkConnector.getInstance().getAPI().getOperatorService().stopOperating(uuid);
    }

    /**
     * Add handler run on connect
     * @param runnable Runnable
     * @return ClientConnection
     */
    public ClientConnection addOnConnectHandler(Runnable runnable) {
        onConnect.add(runnable);
        return this;
    }

    /**
     * Add handler run on disconnect
     * @param runnable Runnable
     * @return ClientConnection
     */
    public ClientConnection addOnDisconnectHandler(Runnable runnable) {
        onDisconnect.add(runnable);
        return this;
    }


}
